//
//  db++.h  - simple database engine in C++
//  Private declarations.
//

//------------------------------------------------------------------------------

class dbField
{
public:
   char*	name;
   virtual int  Set( char* s ) = 0;
   virtual int	Clear( void ) = 0;
   virtual int  Set( char* arg, char* s ) { return 1; };
   virtual int  Write( char* s ) = 0;
   virtual int  Filter( dbField* f ) = 0;
		dbField( char* n ) { name = strdup( n ); };
};

//------------------------------------------------------------------------------

class dbInteger : public dbField
{
private:
   long int     i;      // value
   DBCompar     c;      // comparison function
   int		Compare( dbInteger* q );
public:
		dbInteger( char* n ) : dbField( n )
		{ i = 0; c = C_GE; };
   virtual int  Set( char* s )		{ i = atol( s ); };
   virtual int  Clear( void )		{ i = 0; };
   virtual int  Write( char* s )	{ sprintf( s, "%ld", i ); };
   virtual int  Filter( dbField* f );
   virtual int	Set( char* arg, char* val );
};

//------------------------------------------------------------------------------

class dbString : public dbField
{
private:
   char		s[256];      // value
public:
		dbString( char* n ) : dbField( n )
		{ s[0] = 0; };
   virtual int  Set( char* S )		{ sprintf( s, "%s", S ); };
   virtual int  Clear( void )		{ s[0] = 0; };
   virtual int  Write( char* S )	{ sprintf( S, "%s", s ); };
   virtual int  Filter( dbField* f );
};

//------------------------------------------------------------------------------

class dbDate : public dbField
{
private:
   char		d[7];	// YYMMDD0
   DBCompar     c;      // comparison function
   int		Compare( dbDate* q );
public:
		dbDate( char *n ) : dbField( n )
		{ c = C_NONE; d[6] = 0; };
   virtual int  Set( char* s );
   virtual int  Clear( void )		{ ; };
   virtual int  Write( char* s );
   virtual int  Filter( dbField* f );
   virtual int	Set( char* arg, char* val );
};

//------------------------------------------------------------------------------

struct node {
   char*	key;
   char*	label;
};

//------------------------------------------------------------------------------

class dbOneOne : public dbField
{
private:
   char*	values[32];
   int		length;
   char*	root;
public:
		dbOneOne( char* n ) : dbField( n )
					{ length = 0; };
   virtual int  Set( char* s );
   virtual int  Clear( void )		{ length = 0; };
   virtual int  Write( char* s );
   virtual int  Filter( dbField* f );
};

//------------------------------------------------------------------------------

class dbOneMany : public dbField
{
private:
   char*	values[32];
   int		length;
   int		and;	// should values be ANDed or ORed?
   FILE*	fp;
public:
		dbOneMany( char* n ) : dbField( n )
					{ length = 0; and = 0; };
   virtual int  Set( char* s );
   virtual int  Clear( void )		{ length = 0; };
   virtual int  Write( char* s );
   virtual int  Filter( dbField* f );
   virtual int	Set( char* arg, char* val );
};

//------------------------------------------------------------------------------

class dbEngine
{
private:
   dbField*	data[255];
   dbField*	query[255];
   int		nfields;
public:
   long int	seconds;
   long int	bytesIn;
   long int	bytesOut;

		dbEngine( void ) { nfields = 0; };
   int		AddField( DBField* f );
   int		AddFields( DBField f[] )
		{
		   for( int i = 0; f[i].type != LAST; i++ )
		      AddField( &f[i] );
		   return 1;
		};
   int		FindField( char* n )
		{
		   int i;
		   for( i = 0; i < nfields; i++ )
		      if( strcmp( n, query[i]->name ) == 0 )
		         return i;
		   return -1;
		};
   int		SetField( char* n, char* val, int qry );
   int		ClearRecord( int qry )
		{
		   int i;
		   for( i = 0; i < nfields; i++ )
		   {
		      if( qry ) query[i]->Clear();
		      else data[i]->Clear();
		   }
		};
   int		SetRecord( char* s, int qry );
   int		Write( int i, char* s, int qry )
		{
		   if( qry ) query[i]->Write( s );
		   else data[i]->Write( s );
		   return 1;
		};
   int		Write( char* s, int qry )
		{
		   for( int i = 0; i < nfields; i++ )
		   {
		      if( i ) sprintf( s+strlen(s), "|" );
		      Write( i, s+strlen(s), qry );
		   }
		   return i;
		};
   int		Filter( void )
		{
		   for( int i = 0; i < nfields; i++ )
		      if( data[i]->Filter( query[i] ) == -1 )
		         return 0;
		   return 1;
		};
   int		Filter( int i )
		{
		   return data[i]->Filter( query[i] );
		};
};
